"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var state_machine_1 = __importDefault(require("./state-machine"));
exports.EditableStatus = state_machine_1.Status;
var invariant_1 = __importDefault(require("invariant"));
var prop_types_1 = __importDefault(require("prop-types"));
var react_1 = __importDefault(require("react"));
var make_cancelable_1 = __importDefault(require("./make-cancelable"));
exports.EditableStatusType = prop_types_1.default.oneOf(Object.keys(state_machine_1.Status));
function getValue(props, state) {
    return state.status === state_machine_1.Status.PRESENTING
        ? props.value
        : state.value;
}
var Editable = /** @class */ (function (_super) {
    __extends(Editable, _super);
    function Editable() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            status: state_machine_1.Status.PRESENTING,
            value: undefined
        };
        _this.handleStart = function () {
            _this.setState(function (state) {
                return state_machine_1.default(state.status, state_machine_1.Action.START, _this.props.value);
            });
        };
        _this.handleChange = function (nextValue) {
            _this.setState(function (state) { return state_machine_1.default(state.status, state_machine_1.Action.CHANGE, nextValue); });
        };
        _this.handleCancel = function () {
            var onCancel = _this.props.onCancel;
            var _a = _this.state, status = _a.status, value = _a.value;
            if (typeof onCancel === "function" && status === state_machine_1.Status.EDITING) {
                onCancel(value);
            }
            _this.setState(function (state) { return state_machine_1.default(state.status, state_machine_1.Action.CANCEL); });
        };
        _this.handleCommit = function (message) {
            var onCommit = _this.props.onCommit;
            invariant_1.default(_this.state.status !== state_machine_1.Status.COMMITTING, "React Editable cannot commit while commiting");
            _this.setState(function (state) {
                return state_machine_1.default(state.status, state_machine_1.Action.COMMIT, getValue(_this.props, _this.state));
            });
            if (typeof onCommit === "function") {
                // TODO: find a way to test this async behavior (enzyme makes setState synchronous)
                // May have just started and not yet updated state
                var maybeCommitPromise = onCommit(message, getValue(_this.props, _this.state));
                if (maybeCommitPromise && maybeCommitPromise.then) {
                    _this.commitPromise = make_cancelable_1.default(maybeCommitPromise);
                    return _this.commitPromise
                        .then(function () {
                        return _this.setState(function (state) { return state_machine_1.default(state.status, state_machine_1.Action.SUCCESS); });
                    })
                        .catch(function (response) {
                        if (!response || !response.isCanceled) {
                            _this.setState(function (state) { return state_machine_1.default(state.status, state_machine_1.Action.FAIL); });
                        }
                    })
                        .then(function () { return (_this.commitPromise = undefined); });
                }
            }
            return new Promise(function (resolve) {
                _this.setState(function (state) { return state_machine_1.default(state.status, state_machine_1.Action.SUCCESS); }, resolve);
            });
        };
        return _this;
    }
    Editable.prototype.componentWillUnmount = function () {
        if (this.commitPromise) {
            this.commitPromise.cancel();
        }
    };
    Editable.prototype.render = function () {
        var status = this.state.status;
        var children = this.props.children;
        return children({
            onCancel: this.handleCancel,
            onChange: this.handleChange,
            onCommit: this.handleCommit,
            onStart: this.handleStart,
            status: status,
            value: getValue(this.props, this.state)
        });
    };
    Editable.displayName = "Editable";
    Editable.propTypes = {
        children: prop_types_1.default.func,
        onCommit: prop_types_1.default.func,
        value: prop_types_1.default.any
    };
    Editable.defaultProps = {
        children: function () { return null; }
    };
    return Editable;
}(react_1.default.Component));
exports.default = Editable;
